"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
const Helpers = __importStar(require("./Helpers"));
const postReceiveFunction1 = async function (items, response) {
    items.forEach((item) => (item.json1 = { success: true }));
    return items;
};
const preSendFunction1 = async function (requestOptions) {
    requestOptions.headers = (requestOptions.headers || {});
    requestOptions.headers.addedIn = 'preSendFunction1';
    return requestOptions;
};
describe('RoutingNode', () => {
    describe('getRequestOptionsFromParameters', () => {
        const tests = [
            {
                description: 'single parameter, only send defined, fixed value',
                input: {
                    nodeParameters: {},
                    nodeTypeProperties: {
                        displayName: 'Email',
                        name: 'email',
                        type: 'string',
                        routing: {
                            send: {
                                property: 'toEmail',
                                type: 'body',
                                value: 'fixedValue',
                            },
                        },
                        default: '',
                    },
                },
                output: {
                    options: {
                        qs: {},
                        body: {
                            toEmail: 'fixedValue',
                        },
                    },
                    preSend: [],
                    postReceive: [],
                    requestOperations: {},
                },
            },
            {
                description: 'single parameter, only send defined, using expression',
                input: {
                    nodeParameters: {
                        email: 'test@test.com',
                    },
                    nodeTypeProperties: {
                        displayName: 'Email',
                        name: 'email',
                        type: 'string',
                        routing: {
                            send: {
                                property: 'toEmail',
                                type: 'body',
                                value: '={{$value.toUpperCase()}}',
                            },
                        },
                        default: '',
                    },
                },
                output: {
                    options: {
                        qs: {},
                        body: {
                            toEmail: 'TEST@TEST.COM',
                        },
                    },
                    preSend: [],
                    postReceive: [],
                    requestOperations: {},
                },
            },
            {
                description: 'single parameter, send and operations defined, fixed value',
                input: {
                    nodeParameters: {},
                    nodeTypeProperties: {
                        displayName: 'Email',
                        name: 'email',
                        type: 'string',
                        routing: {
                            send: {
                                property: 'toEmail',
                                type: 'body',
                                value: 'fixedValue',
                            },
                            operations: {
                                pagination: {
                                    type: 'offset',
                                    properties: {
                                        limitParameter: 'limit',
                                        offsetParameter: 'offset',
                                        pageSize: 10,
                                        rootProperty: 'data',
                                        type: 'body',
                                    },
                                },
                            },
                        },
                        default: '',
                    },
                },
                output: {
                    options: {
                        qs: {},
                        body: {
                            toEmail: 'fixedValue',
                        },
                    },
                    preSend: [],
                    postReceive: [],
                    requestOperations: {
                        pagination: {
                            type: 'offset',
                            properties: {
                                limitParameter: 'limit',
                                offsetParameter: 'offset',
                                pageSize: 10,
                                rootProperty: 'data',
                                type: 'body',
                            },
                        },
                    },
                },
            },
            {
                description: 'mutliple parameters, complex example with everything',
                input: {
                    nodeParameters: {
                        multipleFields: {
                            value1: 'v1',
                            value2: 'v2',
                            value3: 'v3',
                            value4: 4,
                            lowerLevel: {
                                lowLevelValue1: 1,
                                lowLevelValue2: 'llv2',
                            },
                            customPropertiesSingle1: {
                                property: {
                                    name: 'cSName1',
                                    value: 'cSValue1',
                                },
                            },
                            customPropertiesMulti: {
                                property0: [
                                    {
                                        name: 'cM0Name1',
                                        value: 'cM0Value1',
                                    },
                                    {
                                        name: 'cM0Name2',
                                        value: 'cM0Value2',
                                    },
                                ],
                                property1: [
                                    {
                                        name: 'cM1Name2',
                                        value: 'cM1Value2',
                                    },
                                    {
                                        name: 'cM1Name2',
                                        value: 'cM1Value2',
                                    },
                                ],
                            },
                        },
                    },
                    nodeTypeProperties: {
                        displayName: 'Multiple Fields',
                        name: 'multipleFields',
                        type: 'collection',
                        placeholder: 'Add Field',
                        routing: {
                            request: {
                                method: 'GET',
                                url: '/destination1',
                            },
                            operations: {
                                pagination: {
                                    type: 'offset',
                                    properties: {
                                        limitParameter: 'limit1',
                                        offsetParameter: 'offset1',
                                        pageSize: 1,
                                        rootProperty: 'data1',
                                        type: 'body',
                                    },
                                },
                            },
                            output: {
                                maxResults: 10,
                                postReceive: [postReceiveFunction1],
                            },
                        },
                        default: {},
                        options: [
                            {
                                displayName: 'Value 1',
                                name: 'value1',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'value1',
                                        type: 'body',
                                    },
                                },
                                default: '',
                            },
                            {
                                displayName: 'Value 2',
                                name: 'value2',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'topLevel.value2',
                                        propertyInDotNotation: false,
                                        type: 'body',
                                        preSend: [preSendFunction1],
                                    },
                                },
                                default: '',
                            },
                            {
                                displayName: 'Value 3',
                                name: 'value3',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'lowerLevel.value3',
                                        type: 'body',
                                    },
                                },
                                default: '',
                            },
                            {
                                displayName: 'Value 4',
                                name: 'value4',
                                type: 'number',
                                default: 0,
                                routing: {
                                    send: {
                                        property: 'value4',
                                        type: 'query',
                                    },
                                    output: {
                                        maxResults: '={{$value}}',
                                    },
                                    operations: {
                                        pagination: {
                                            type: 'offset',
                                            properties: {
                                                limitParameter: 'limit100',
                                                offsetParameter: 'offset100',
                                                pageSize: 100,
                                                rootProperty: 'data100',
                                                type: 'query',
                                            },
                                        },
                                    },
                                },
                            },
                            {
                                displayName: 'Value 5',
                                name: 'value5',
                                type: 'number',
                                displayOptions: {
                                    show: {
                                        value4: [1],
                                    },
                                },
                                default: 5,
                                routing: {
                                    send: {
                                        property: 'value5',
                                        type: 'query',
                                    },
                                    operations: {
                                        pagination: {
                                            type: 'offset',
                                            properties: {
                                                limitParameter: 'limit10',
                                                offsetParameter: 'offset10',
                                                pageSize: 10,
                                                rootProperty: 'data10',
                                                type: 'body',
                                            },
                                        },
                                    },
                                },
                            },
                            {
                                displayName: 'Lower Level',
                                name: 'lowerLevel',
                                type: 'collection',
                                placeholder: 'Add Field',
                                default: {},
                                options: [
                                    {
                                        displayName: 'Low Level Value1',
                                        name: 'lowLevelValue1',
                                        type: 'number',
                                        default: 0,
                                        routing: {
                                            send: {
                                                property: 'llvalue1',
                                                type: 'query',
                                            },
                                        },
                                    },
                                    {
                                        displayName: 'Low Level Value2',
                                        name: 'lowLevelValue2',
                                        type: 'string',
                                        default: '',
                                        routing: {
                                            send: {
                                                property: 'llvalue2',
                                                type: 'query',
                                                preSend: [preSendFunction1],
                                            },
                                            output: {
                                                postReceive: [
                                                    {
                                                        type: 'rootProperty',
                                                        properties: {
                                                            property: 'data',
                                                        },
                                                    },
                                                ],
                                            },
                                        },
                                    },
                                ],
                            },
                            {
                                displayName: 'Custom Properties1 (single)',
                                name: 'customPropertiesSingle1',
                                placeholder: 'Add Custom Property',
                                type: 'fixedCollection',
                                default: {},
                                options: [
                                    {
                                        name: 'property',
                                        displayName: 'Property',
                                        values: [
                                            {
                                                displayName: 'Property Name',
                                                name: 'name',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    request: {
                                                        method: 'POST',
                                                        url: '=/{{$value}}',
                                                    },
                                                    send: {
                                                        property: 'single-customValues.name',
                                                    },
                                                },
                                            },
                                            {
                                                displayName: 'Property Value',
                                                name: 'value',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    send: {
                                                        property: 'single-customValues.value',
                                                    },
                                                },
                                            },
                                        ],
                                    },
                                ],
                            },
                            {
                                displayName: 'Custom Properties (multi)',
                                name: 'customPropertiesMulti',
                                placeholder: 'Add Custom Property',
                                type: 'fixedCollection',
                                typeOptions: {
                                    multipleValues: true,
                                },
                                default: {},
                                options: [
                                    {
                                        name: 'property0',
                                        displayName: 'Property0',
                                        values: [
                                            {
                                                displayName: 'Property Name',
                                                name: 'name',
                                                type: 'string',
                                                default: '',
                                                description: 'Name of the property to set.',
                                            },
                                            {
                                                displayName: 'Property Value',
                                                name: 'value',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    send: {
                                                        property: '=customMulti0.{{$parent.name}}',
                                                        type: 'body',
                                                    },
                                                },
                                                description: 'Value of the property to set.',
                                            },
                                        ],
                                    },
                                    {
                                        name: 'property1',
                                        displayName: 'Property1',
                                        values: [
                                            {
                                                displayName: 'Property Name',
                                                name: 'name',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    send: {
                                                        property: '=customMulti1[{{$index}}].name',
                                                        type: 'body',
                                                    },
                                                },
                                            },
                                            {
                                                displayName: 'Property Value',
                                                name: 'value',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    send: {
                                                        property: '=customMulti1[{{$index}}].value',
                                                        type: 'body',
                                                    },
                                                },
                                            },
                                        ],
                                    },
                                ],
                            },
                        ],
                    },
                },
                output: {
                    maxResults: 4,
                    options: {
                        method: 'POST',
                        url: '/cSName1',
                        qs: {
                            value4: 4,
                            llvalue1: 1,
                            llvalue2: 'llv2',
                            'single-customValues': {
                                name: 'cSName1',
                                value: 'cSValue1',
                            },
                        },
                        body: {
                            value1: 'v1',
                            'topLevel.value2': 'v2',
                            lowerLevel: {
                                value3: 'v3',
                            },
                            customMulti0: {
                                cM0Name1: 'cM0Value1',
                                cM0Name2: 'cM0Value2',
                            },
                            customMulti1: [
                                {
                                    name: 'cM1Name2',
                                    value: 'cM1Value2',
                                },
                                {
                                    name: 'cM1Name2',
                                    value: 'cM1Value2',
                                },
                            ],
                        },
                    },
                    preSend: [preSendFunction1, preSendFunction1],
                    postReceive: [
                        {
                            actions: [postReceiveFunction1],
                            data: {
                                parameterValue: {
                                    value1: 'v1',
                                    value2: 'v2',
                                    value3: 'v3',
                                    value4: 4,
                                    lowerLevel: {
                                        lowLevelValue1: 1,
                                        lowLevelValue2: 'llv2',
                                    },
                                    customPropertiesSingle1: {
                                        property: {
                                            name: 'cSName1',
                                            value: 'cSValue1',
                                        },
                                    },
                                    customPropertiesMulti: {
                                        property0: [
                                            {
                                                name: 'cM0Name1',
                                                value: 'cM0Value1',
                                            },
                                            {
                                                name: 'cM0Name2',
                                                value: 'cM0Value2',
                                            },
                                        ],
                                        property1: [
                                            {
                                                name: 'cM1Name2',
                                                value: 'cM1Value2',
                                            },
                                            {
                                                name: 'cM1Name2',
                                                value: 'cM1Value2',
                                            },
                                        ],
                                    },
                                },
                            },
                        },
                        {
                            actions: [
                                {
                                    type: 'rootProperty',
                                    properties: {
                                        property: 'data',
                                    },
                                },
                            ],
                            data: {
                                parameterValue: 'llv2',
                            },
                        },
                    ],
                    requestOperations: {
                        pagination: {
                            type: 'offset',
                            properties: {
                                limitParameter: 'limit100',
                                offsetParameter: 'offset100',
                                pageSize: 100,
                                rootProperty: 'data100',
                                type: 'query',
                            },
                        },
                    },
                },
            },
        ];
        const nodeTypes = Helpers.NodeTypes();
        const node = {
            parameters: {},
            name: 'test',
            type: 'test.set',
            typeVersion: 1,
            position: [0, 0],
        };
        const mode = 'internal';
        const runIndex = 0;
        const itemIndex = 0;
        const connectionInputData = [];
        const runExecutionData = { resultData: { runData: {} } };
        const additionalData = Helpers.WorkflowExecuteAdditionalData();
        const path = '';
        const nodeType = nodeTypes.getByNameAndVersion(node.type);
        const workflowData = {
            nodes: [node],
            connections: {},
        };
        for (const testData of tests) {
            test(testData.description, () => {
                node.parameters = testData.input.nodeParameters;
                nodeType.description.properties = [testData.input.nodeTypeProperties];
                const workflow = new src_1.Workflow({
                    nodes: workflowData.nodes,
                    connections: workflowData.connections,
                    active: false,
                    nodeTypes,
                });
                const routingNode = new src_1.RoutingNode(workflow, node, connectionInputData, runExecutionData !== null && runExecutionData !== void 0 ? runExecutionData : null, additionalData, mode);
                const executeSingleFunctions = Helpers.getExecuteSingleFunctions(workflow, runExecutionData, runIndex, connectionInputData, {}, node, itemIndex, additionalData, mode);
                const result = routingNode.getRequestOptionsFromParameters(executeSingleFunctions, testData.input.nodeTypeProperties, itemIndex, runIndex, path, {});
                expect(result).toEqual(testData.output);
            });
        }
    });
    describe('runNode', () => {
        const tests = [
            {
                description: 'single parameter, only send defined, fixed value, using requestDefaults',
                input: {
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'Email',
                                name: 'email',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'toEmail',
                                        type: 'body',
                                        value: 'fixedValue',
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                    node: {
                        parameters: {},
                    },
                },
                output: [
                    [
                        {
                            json: {
                                headers: {},
                                statusCode: 200,
                                requestOptions: {
                                    url: '/test-url',
                                    qs: {},
                                    body: {
                                        toEmail: 'fixedValue',
                                    },
                                    baseURL: 'http://127.0.0.1:5678',
                                    returnFullResponse: true,
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, only send defined, fixed value, using requestDefaults',
                input: {
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'Email',
                                name: 'email',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'toEmail',
                                        type: 'body',
                                        value: 'fixedValue',
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                    node: {
                        parameters: {},
                    },
                },
                output: [
                    [
                        {
                            json: {
                                headers: {},
                                statusCode: 200,
                                requestOptions: {
                                    url: '/test-url',
                                    qs: {},
                                    body: {
                                        toEmail: 'fixedValue',
                                    },
                                    baseURL: 'http://127.0.0.1:5678',
                                    returnFullResponse: true,
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, only send defined, using expression, using requestDefaults with overwrite',
                input: {
                    node: {
                        parameters: {
                            email: 'test@test.com',
                        },
                    },
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'Email',
                                name: 'email',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'toEmail',
                                        type: 'body',
                                        value: '={{$value.toUpperCase()}}',
                                    },
                                    request: {
                                        url: '/overwritten',
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                },
                output: [
                    [
                        {
                            json: {
                                headers: {},
                                statusCode: 200,
                                requestOptions: {
                                    url: '/overwritten',
                                    qs: {},
                                    body: {
                                        toEmail: 'TEST@TEST.COM',
                                    },
                                    baseURL: 'http://127.0.0.1:5678',
                                    returnFullResponse: true,
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, only send defined, using expression, using requestDefaults with overwrite and expressions',
                input: {
                    node: {
                        parameters: {
                            endpoint: 'custom-overwritten',
                        },
                    },
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'Endpoint',
                                name: 'endpoint',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: '={{"theProperty"}}',
                                        type: 'body',
                                        value: '={{$value}}',
                                    },
                                    request: {
                                        url: '=/{{$value}}',
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                },
                output: [
                    [
                        {
                            json: {
                                headers: {},
                                statusCode: 200,
                                requestOptions: {
                                    url: '/custom-overwritten',
                                    qs: {},
                                    body: {
                                        theProperty: 'custom-overwritten',
                                    },
                                    baseURL: 'http://127.0.0.1:5678',
                                    returnFullResponse: true,
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, send and operations defined, fixed value with pagination',
                input: {
                    node: {
                        parameters: {},
                    },
                    nodeType: {
                        properties: [
                            {
                                displayName: 'Email',
                                name: 'email',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'toEmail',
                                        type: 'body',
                                        value: 'fixedValue',
                                        paginate: true,
                                    },
                                    operations: {
                                        pagination: {
                                            type: 'offset',
                                            properties: {
                                                limitParameter: 'limit',
                                                offsetParameter: 'offset',
                                                pageSize: 10,
                                                type: 'body',
                                            },
                                        },
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                },
                output: [
                    [
                        {
                            json: {
                                headers: {},
                                statusCode: 200,
                                requestOptions: {
                                    qs: {},
                                    body: {
                                        toEmail: 'fixedValue',
                                        limit: 10,
                                        offset: 10,
                                    },
                                    returnFullResponse: true,
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'mutliple parameters, complex example with everything',
                input: {
                    node: {
                        parameters: {
                            multipleFields: {
                                value1: 'v1',
                                value2: 'v2',
                                value3: 'v3',
                                value4: 4,
                                lowerLevel: {
                                    lowLevelValue1: 1,
                                    lowLevelValue2: 'llv2',
                                },
                                customPropertiesSingle1: {
                                    property: {
                                        name: 'cSName1',
                                        value: 'cSValue1',
                                    },
                                },
                                customPropertiesMulti: {
                                    property0: [
                                        {
                                            name: 'cM0Name1',
                                            value: 'cM0Value1',
                                        },
                                        {
                                            name: 'cM0Name2',
                                            value: 'cM0Value2',
                                        },
                                    ],
                                    property1: [
                                        {
                                            name: 'cM1Name2',
                                            value: 'cM1Value2',
                                        },
                                        {
                                            name: 'cM1Name2',
                                            value: 'cM1Value2',
                                        },
                                    ],
                                },
                            },
                        },
                    },
                    nodeType: {
                        properties: [
                            {
                                displayName: 'Multiple Fields',
                                name: 'multipleFields',
                                type: 'collection',
                                placeholder: 'Add Field',
                                routing: {
                                    request: {
                                        method: 'GET',
                                        url: '/destination1',
                                    },
                                    operations: {
                                        pagination: {
                                            type: 'offset',
                                            properties: {
                                                limitParameter: 'limit1',
                                                offsetParameter: 'offset1',
                                                pageSize: 1,
                                                rootProperty: 'data1',
                                                type: 'body',
                                            },
                                        },
                                    },
                                    output: {
                                        maxResults: 10,
                                        postReceive: [postReceiveFunction1],
                                    },
                                },
                                default: {},
                                options: [
                                    {
                                        displayName: 'Value 1',
                                        name: 'value1',
                                        type: 'string',
                                        routing: {
                                            send: {
                                                property: 'value1',
                                                type: 'body',
                                            },
                                        },
                                        default: '',
                                    },
                                    {
                                        displayName: 'Value 2',
                                        name: 'value2',
                                        type: 'string',
                                        routing: {
                                            send: {
                                                property: 'topLevel.value2',
                                                propertyInDotNotation: false,
                                                type: 'body',
                                                preSend: [preSendFunction1],
                                            },
                                        },
                                        default: '',
                                    },
                                    {
                                        displayName: 'Value 3',
                                        name: 'value3',
                                        type: 'string',
                                        routing: {
                                            send: {
                                                property: 'lowerLevel.value3',
                                                type: 'body',
                                            },
                                        },
                                        default: '',
                                    },
                                    {
                                        displayName: 'Value 4',
                                        name: 'value4',
                                        type: 'number',
                                        default: 0,
                                        routing: {
                                            send: {
                                                property: 'value4',
                                                type: 'query',
                                            },
                                            output: {
                                                maxResults: '={{$value}}',
                                            },
                                            operations: {
                                                pagination: {
                                                    type: 'offset',
                                                    properties: {
                                                        limitParameter: 'limit100',
                                                        offsetParameter: 'offset100',
                                                        pageSize: 100,
                                                        rootProperty: 'data100',
                                                        type: 'query',
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    {
                                        displayName: 'Value 5',
                                        name: 'value5',
                                        type: 'number',
                                        displayOptions: {
                                            show: {
                                                value4: [1],
                                            },
                                        },
                                        default: 5,
                                        routing: {
                                            send: {
                                                property: 'value5',
                                                type: 'query',
                                            },
                                            operations: {
                                                pagination: {
                                                    type: 'offset',
                                                    properties: {
                                                        limitParameter: 'limit10',
                                                        offsetParameter: 'offset10',
                                                        pageSize: 10,
                                                        rootProperty: 'data10',
                                                        type: 'body',
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    {
                                        displayName: 'Lower Level',
                                        name: 'lowerLevel',
                                        type: 'collection',
                                        placeholder: 'Add Field',
                                        default: {},
                                        options: [
                                            {
                                                displayName: 'Low Level Value1',
                                                name: 'lowLevelValue1',
                                                type: 'number',
                                                default: 0,
                                                routing: {
                                                    send: {
                                                        property: 'llvalue1',
                                                        type: 'query',
                                                    },
                                                },
                                            },
                                            {
                                                displayName: 'Low Level Value2',
                                                name: 'lowLevelValue2',
                                                type: 'string',
                                                default: '',
                                                routing: {
                                                    send: {
                                                        property: 'llvalue2',
                                                        type: 'query',
                                                        preSend: [preSendFunction1],
                                                    },
                                                    output: {
                                                        postReceive: [
                                                            {
                                                                type: 'rootProperty',
                                                                properties: {
                                                                    property: 'requestOptions',
                                                                },
                                                            },
                                                        ],
                                                    },
                                                },
                                            },
                                        ],
                                    },
                                    {
                                        displayName: 'Custom Properties1 (single)',
                                        name: 'customPropertiesSingle1',
                                        placeholder: 'Add Custom Property',
                                        type: 'fixedCollection',
                                        default: {},
                                        options: [
                                            {
                                                name: 'property',
                                                displayName: 'Property',
                                                values: [
                                                    {
                                                        displayName: 'Property Name',
                                                        name: 'name',
                                                        type: 'string',
                                                        default: '',
                                                        routing: {
                                                            request: {
                                                                method: 'POST',
                                                                url: '=/{{$value}}',
                                                            },
                                                            send: {
                                                                property: 'single-customValues.name',
                                                            },
                                                        },
                                                    },
                                                    {
                                                        displayName: 'Property Value',
                                                        name: 'value',
                                                        type: 'string',
                                                        default: '',
                                                        routing: {
                                                            send: {
                                                                property: 'single-customValues.value',
                                                            },
                                                        },
                                                    },
                                                ],
                                            },
                                        ],
                                    },
                                    {
                                        displayName: 'Custom Properties (multi)',
                                        name: 'customPropertiesMulti',
                                        placeholder: 'Add Custom Property',
                                        type: 'fixedCollection',
                                        typeOptions: {
                                            multipleValues: true,
                                        },
                                        default: {},
                                        options: [
                                            {
                                                name: 'property0',
                                                displayName: 'Property0',
                                                values: [
                                                    {
                                                        displayName: 'Property Name',
                                                        name: 'name',
                                                        type: 'string',
                                                        default: '',
                                                        description: 'Name of the property to set.',
                                                    },
                                                    {
                                                        displayName: 'Property Value',
                                                        name: 'value',
                                                        type: 'string',
                                                        default: '',
                                                        routing: {
                                                            send: {
                                                                property: '=customMulti0.{{$parent.name}}',
                                                                type: 'body',
                                                            },
                                                        },
                                                        description: 'Value of the property to set.',
                                                    },
                                                ],
                                            },
                                            {
                                                name: 'property1',
                                                displayName: 'Property1',
                                                values: [
                                                    {
                                                        displayName: 'Property Name',
                                                        name: 'name',
                                                        type: 'string',
                                                        default: '',
                                                        routing: {
                                                            send: {
                                                                property: '=customMulti1[{{$index}}].name',
                                                                type: 'body',
                                                            },
                                                        },
                                                    },
                                                    {
                                                        displayName: 'Property Value',
                                                        name: 'value',
                                                        type: 'string',
                                                        default: '',
                                                        routing: {
                                                            send: {
                                                                property: '=customMulti1[{{$index}}].value',
                                                                type: 'body',
                                                            },
                                                        },
                                                    },
                                                ],
                                            },
                                        ],
                                    },
                                ],
                            },
                        ],
                    },
                },
                output: [
                    [
                        {
                            json: {
                                url: '/cSName1',
                                qs: {
                                    value4: 4,
                                    llvalue1: 1,
                                    llvalue2: 'llv2',
                                    'single-customValues': {
                                        name: 'cSName1',
                                        value: 'cSValue1',
                                    },
                                },
                                body: {
                                    value1: 'v1',
                                    'topLevel.value2': 'v2',
                                    lowerLevel: {
                                        value3: 'v3',
                                    },
                                    customMulti0: {
                                        cM0Name1: 'cM0Value1',
                                        cM0Name2: 'cM0Value2',
                                    },
                                    customMulti1: [
                                        {
                                            name: 'cM1Name2',
                                            value: 'cM1Value2',
                                        },
                                        {
                                            name: 'cM1Name2',
                                            value: 'cM1Value2',
                                        },
                                    ],
                                },
                                method: 'POST',
                                headers: {
                                    addedIn: 'preSendFunction1',
                                },
                                returnFullResponse: true,
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, postReceive: set',
                input: {
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'JSON Data',
                                name: 'jsonData',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'jsonData',
                                        type: 'body',
                                    },
                                    output: {
                                        postReceive: [
                                            {
                                                type: 'set',
                                                properties: {
                                                    value: '={{ { "value": $value, "response": $response } }}',
                                                },
                                            },
                                        ],
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                    node: {
                        parameters: {
                            jsonData: {
                                root: [
                                    {
                                        name: 'Jim',
                                        age: 34,
                                    },
                                    {
                                        name: 'James',
                                        age: 44,
                                    },
                                ],
                            },
                        },
                    },
                },
                output: [
                    [
                        {
                            json: {
                                value: {
                                    root: [
                                        {
                                            name: 'Jim',
                                            age: 34,
                                        },
                                        {
                                            name: 'James',
                                            age: 44,
                                        },
                                    ],
                                },
                                response: {
                                    body: {
                                        headers: {},
                                        statusCode: 200,
                                        requestOptions: {
                                            qs: {},
                                            body: {
                                                jsonData: {
                                                    root: [
                                                        {
                                                            name: 'Jim',
                                                            age: 34,
                                                        },
                                                        {
                                                            name: 'James',
                                                            age: 44,
                                                        },
                                                    ],
                                                },
                                            },
                                            baseURL: 'http://127.0.0.1:5678',
                                            url: '/test-url',
                                            returnFullResponse: true,
                                        },
                                    },
                                },
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, postReceive: rootProperty',
                input: {
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'JSON Data',
                                name: 'jsonData',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'jsonData',
                                        type: 'body',
                                    },
                                    output: {
                                        postReceive: [
                                            {
                                                type: 'rootProperty',
                                                properties: {
                                                    property: 'requestOptions',
                                                },
                                            },
                                            {
                                                type: 'rootProperty',
                                                properties: {
                                                    property: 'body.jsonData.root',
                                                },
                                            },
                                        ],
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                    node: {
                        parameters: {
                            jsonData: {
                                root: [
                                    {
                                        name: 'Jim',
                                        age: 34,
                                    },
                                    {
                                        name: 'James',
                                        age: 44,
                                    },
                                ],
                            },
                        },
                    },
                },
                output: [
                    [
                        {
                            json: {
                                name: 'Jim',
                                age: 34,
                            },
                        },
                        {
                            json: {
                                name: 'James',
                                age: 44,
                            },
                        },
                    ],
                ],
            },
            {
                description: 'single parameter, mutliple postReceive: rootProperty, setKeyValue, sort',
                input: {
                    nodeType: {
                        requestDefaults: {
                            baseURL: 'http://127.0.0.1:5678',
                            url: '/test-url',
                        },
                        properties: [
                            {
                                displayName: 'JSON Data',
                                name: 'jsonData',
                                type: 'string',
                                routing: {
                                    send: {
                                        property: 'jsonData',
                                        type: 'body',
                                    },
                                    output: {
                                        postReceive: [
                                            {
                                                type: 'rootProperty',
                                                properties: {
                                                    property: 'requestOptions.body.jsonData.root',
                                                },
                                            },
                                            {
                                                type: 'setKeyValue',
                                                properties: {
                                                    display1: '={{$responseItem.name}} ({{$responseItem.age}})',
                                                    display2: '={{$responseItem.name}} is {{$responseItem.age}}',
                                                },
                                            },
                                            {
                                                type: 'sort',
                                                properties: {
                                                    key: 'display1',
                                                },
                                            },
                                        ],
                                    },
                                },
                                default: '',
                            },
                        ],
                    },
                    node: {
                        parameters: {
                            jsonData: {
                                root: [
                                    {
                                        name: 'Jim',
                                        age: 34,
                                    },
                                    {
                                        name: 'James',
                                        age: 44,
                                    },
                                ],
                            },
                        },
                    },
                },
                output: [
                    [
                        {
                            json: {
                                display1: 'James (44)',
                                display2: 'James is 44',
                            },
                        },
                        {
                            json: {
                                display1: 'Jim (34)',
                                display2: 'Jim is 34',
                            },
                        },
                    ],
                ],
            },
        ];
        const nodeTypes = Helpers.NodeTypes();
        const baseNode = {
            parameters: {},
            name: 'test',
            type: 'test.set',
            typeVersion: 1,
            position: [0, 0],
        };
        const mode = 'internal';
        const runIndex = 0;
        const itemIndex = 0;
        const connectionInputData = [];
        const runExecutionData = { resultData: { runData: {} } };
        const additionalData = Helpers.WorkflowExecuteAdditionalData();
        const nodeType = nodeTypes.getByNameAndVersion(baseNode.type);
        const inputData = {
            main: [
                [
                    {
                        json: {},
                    },
                ],
            ],
        };
        for (const testData of tests) {
            test(testData.description, async () => {
                const node = { ...baseNode, ...testData.input.node };
                const workflowData = {
                    nodes: [node],
                    connections: {},
                };
                nodeType.description = { ...testData.input.nodeType };
                const workflow = new src_1.Workflow({
                    nodes: workflowData.nodes,
                    connections: workflowData.connections,
                    active: false,
                    nodeTypes,
                });
                const routingNode = new src_1.RoutingNode(workflow, node, connectionInputData, runExecutionData !== null && runExecutionData !== void 0 ? runExecutionData : null, additionalData, mode);
                const nodeExecuteFunctions = {
                    getExecuteFunctions: () => {
                        return Helpers.getExecuteFunctions(workflow, runExecutionData, runIndex, connectionInputData, {}, node, itemIndex, additionalData, mode);
                    },
                    getExecuteSingleFunctions: () => {
                        return Helpers.getExecuteSingleFunctions(workflow, runExecutionData, runIndex, connectionInputData, {}, node, itemIndex, additionalData, mode);
                    },
                };
                const result = await routingNode.runNode(inputData, runIndex, nodeType, nodeExecuteFunctions);
                expect(result).toEqual(testData.output);
            });
        }
    });
});
//# sourceMappingURL=RoutingNode.test.js.map